﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace SmartCart
{
    class XmlHandler
    {
        private XmlDocument LoadXML(string path)
        {
            XmlDocument doc = new XmlDocument();
            try
            {
                doc.Load(path);
            }
            catch
            {
                return doc;
            }
            return doc;
        }

        public Configuration XmlToConfiguration()
        {
            string path = @"\Program Files\SmartCart\Configurations.xml";
            XmlDocument xmldoc = LoadXML(path);
            Configuration configuration = new Configuration(xmldoc.DocumentElement.Attributes.GetNamedItem("ActiveList").Value,
                                                            xmldoc.DocumentElement.Attributes.GetNamedItem("Language").Value,
                                                            xmldoc.DocumentElement.Attributes.GetNamedItem("TextSize").Value,
                                                            xmldoc.DocumentElement.Attributes.GetNamedItem("Appearance").Value,
                                                            xmldoc.DocumentElement.Attributes.GetNamedItem("SortBy").Value);
            return configuration;
        }

        public List<Shoplist> XmlToShoplists()
        {
            string path = @"\Program Files\SmartCart\Shoppinglists.xml";
            XmlDocument xmldoc = LoadXML(path);
            List<Shoplist> shoplists = new List<Shoplist>();
            foreach (XmlNode node in xmldoc.DocumentElement.ChildNodes)
            {
                Shoplist shoplist = AddShoplist(node);
                shoplists.Add(shoplist);
            }
            return shoplists;
        }

        public Shoplist XmlToProductlist()
        {
            string path = @"\Program Files\SmartCart\Productlist.xml";
            XmlDocument xmldoc = LoadXML(path);
            Shoplist productlist = new Shoplist("Productlist");
            foreach (XmlNode node in xmldoc.DocumentElement.ChildNodes)
            {
                AddCategory(node, productlist);
            }
            return productlist;
        }

        public void XmlToStatistics()
        {
            string path = @"\Program Files\SmartCart\Statistics.xml";
            XmlDocument xmldoc = LoadXML(path);
            //List<Statistics> statistics = new List<Statistics>();
            foreach (XmlNode node in xmldoc.DocumentElement.ChildNodes)
            {
                AddStatYear(node);
            }
        }

        private void AddStatYear(XmlNode xmlNode)
        {
            if (xmlNode.Attributes != null)
            {
                Statistics.AddYear(xmlNode.Attributes.GetNamedItem("year").Value,
                                   xmlNode.Attributes.GetNamedItem("January").Value,
                                   xmlNode.Attributes.GetNamedItem("February").Value,
                                   xmlNode.Attributes.GetNamedItem("March").Value,
                                   xmlNode.Attributes.GetNamedItem("April").Value,
                                   xmlNode.Attributes.GetNamedItem("May").Value,
                                   xmlNode.Attributes.GetNamedItem("June").Value,
                                   xmlNode.Attributes.GetNamedItem("July").Value,
                                   xmlNode.Attributes.GetNamedItem("August").Value,
                                   xmlNode.Attributes.GetNamedItem("September").Value,
                                   xmlNode.Attributes.GetNamedItem("October").Value,
                                   xmlNode.Attributes.GetNamedItem("November").Value,
                                   xmlNode.Attributes.GetNamedItem("December").Value);
            }
        }

        private Shoplist AddShoplist(XmlNode xmlNode)
        {
            if (xmlNode.Attributes != null)
            {
                Shoplist shoplist = new Shoplist(xmlNode.Attributes.GetNamedItem("id").Value);
                foreach (XmlNode childNode in xmlNode.ChildNodes)
                {
                    AddCategory(childNode, shoplist);
                }
                return shoplist;
            }
            return null;
        }

        private void AddCategory(XmlNode xmlNode, Shoplist shoplist)
        {
            if (xmlNode.Attributes != null)
            {
                Category category = new Category(xmlNode.Attributes.GetNamedItem("id").Value,
                                                 xmlNode.Attributes.GetNamedItem("color").Value,
                                                 xmlNode.Attributes.GetNamedItem("position").Value,
                                                 xmlNode.Attributes.GetNamedItem("image").Value);
                shoplist.AddCategory(category);
                foreach (XmlNode childNode in xmlNode.ChildNodes)
                {
                    AddProduct(childNode, category);
                }
            }
        }

        private void AddProduct(XmlNode xmlNode, Category category)
        {
            if (xmlNode.Attributes != null)
            {
                Product product = new Product(xmlNode.Attributes.GetNamedItem("id").Value,
                                              xmlNode.Attributes.GetNamedItem("amount").Value,
                                              xmlNode.Attributes.GetNamedItem("bought").Value,
                                              xmlNode.Attributes.GetNamedItem("price").Value);
                category.AddProduct(product);
            }
        }

        public void ShoplistsToXml(List<Shoplist> shoplists)
        {
            // Create a new file
            XmlTextWriter textWriter = new XmlTextWriter(@"\Program Files\SmartCart\Shoppinglists.xml", null);
            // Opens the document
            textWriter.WriteStartDocument();
            // Write comments
            textWriter.WriteComment("Sparad av SmartCart " + System.DateTime.Today);
            // Obligatorisk "<Products>"
            textWriter.WriteStartElement("Products");
            // Listor
            foreach (Shoplist shoplist in shoplists)
            {
                textWriter.WriteStartElement("List");
                // List-id
                textWriter.WriteStartAttribute("id");
                textWriter.WriteValue(shoplist.name);
                // Kategorier
                foreach (Category category in shoplist.GetCategories())
                {
                    textWriter.WriteStartElement("Category");
                    textWriter.WriteStartAttribute("id");
                    textWriter.WriteValue(category.name);
                    textWriter.WriteStartAttribute("color");
                    textWriter.WriteValue(category.color);
                    textWriter.WriteStartAttribute("position");
                    textWriter.WriteValue(category.position);
                    textWriter.WriteStartAttribute("image");
                    textWriter.WriteValue(category.image);
                    // Produkter
                    foreach (Product product in category.GetProducts())
                    {
                        textWriter.WriteStartElement("Product");
                        textWriter.WriteStartAttribute("id");
                        textWriter.WriteValue(product.name);
                        textWriter.WriteStartAttribute("amount");
                        textWriter.WriteValue(product.amount);
                        textWriter.WriteStartAttribute("bought");
                        textWriter.WriteValue(product.bought);
                        textWriter.WriteStartAttribute("price");
                        textWriter.WriteValue(product.price);
                        textWriter.WriteEndElement();
                    }
                    textWriter.WriteEndElement();
                }
                textWriter.WriteEndElement();
            }
            // Ends the document.
            textWriter.WriteEndDocument();
            // close writer
            textWriter.Close();
        }

        public void ProductlistToXml(Shoplist productlist)
        {
            // Create a new file
            XmlTextWriter textWriter = new XmlTextWriter(@"\Program Files\SmartCart\Productlist.xml", null);
            // Opens the document
            textWriter.WriteStartDocument();
            // Write comments
            textWriter.WriteComment("Sparad av SmartCart " + System.DateTime.Today);
            // Obligatorisk "<Products>"
            textWriter.WriteStartElement("Products");
            // Kategorier
            foreach (Category category in productlist.GetCategories())
            {
                textWriter.WriteStartElement("Category");
                textWriter.WriteStartAttribute("id");
                textWriter.WriteValue(category.name);
                textWriter.WriteStartAttribute("color");
                textWriter.WriteValue(category.color);
                textWriter.WriteStartAttribute("position");
                textWriter.WriteValue(category.position);
                textWriter.WriteStartAttribute("image");
                textWriter.WriteValue(category.image);
                // Produkter
                foreach (Product product in category.GetProducts())
                {
                    textWriter.WriteStartElement("Product");
                    textWriter.WriteStartAttribute("id");
                    textWriter.WriteValue(product.name);
                    textWriter.WriteStartAttribute("amount");
                    textWriter.WriteValue(product.amount);
                    textWriter.WriteStartAttribute("bought");
                    textWriter.WriteValue(product.bought);
                    textWriter.WriteStartAttribute("price");
                    textWriter.WriteValue(product.price);
                    textWriter.WriteEndElement();
                }
                textWriter.WriteEndElement();
            }
            // Ends the document.
            textWriter.WriteEndDocument();
            // close writer
            textWriter.Close();
        }

        public void ConfigurationToXml(Configuration configuration)
        {
            // Create a new file
            XmlTextWriter textWriter = new XmlTextWriter(@"\Program Files\SmartCart\Configurations.xml", null);
            // Opens the document
            textWriter.WriteStartDocument();
            // Write comments
            textWriter.WriteComment("Sparad av SmartCart " + System.DateTime.Today);
            // Konfiguration
            textWriter.WriteStartElement("Configurations");
            // Språk
            textWriter.WriteStartAttribute("Language");
            textWriter.WriteValue(configuration.language);
            // Textstorlek
            textWriter.WriteStartAttribute("TextSize");
            textWriter.WriteValue(configuration.textSize);
            // Aktiv lista
            textWriter.WriteStartAttribute("ActiveList");
            textWriter.WriteValue(configuration.activeList);
            // Utseende
            textWriter.WriteStartAttribute("Appearance");
            textWriter.WriteValue(configuration.appearance);
            // Sortering
            textWriter.WriteStartAttribute("SortBy");
            textWriter.WriteValue(configuration.sortBy);
            // Avsluta konfiguration
            textWriter.WriteEndElement();
            // close writer
            textWriter.Close();
        }

        public void StatisticsToXml()
        {
            // Create a new file
            XmlTextWriter textWriter = new XmlTextWriter(@"\Program Files\SmartCart\Statistics.xml", null);
            // Opens the document
            textWriter.WriteStartDocument();
            // Write comments
            textWriter.WriteComment("Sparad av SmartCart " + System.DateTime.Today);
            // Obligatorisk "<Statistics>"
            textWriter.WriteStartElement("Statistics");
            // Listor
            foreach (string year in Statistics.years.Keys)
            {
                // Statistik
                textWriter.WriteStartElement("Stats");
                // År
                textWriter.WriteStartAttribute("year");
                textWriter.WriteValue(year);
                // Januari
                textWriter.WriteStartAttribute("January");
                textWriter.WriteValue(Statistics.years[year]["1"]);
                // Februari
                textWriter.WriteStartAttribute("February");
                textWriter.WriteValue(Statistics.years[year]["2"]);
                // Mars
                textWriter.WriteStartAttribute("March");
                textWriter.WriteValue(Statistics.years[year]["3"]);
                // April
                textWriter.WriteStartAttribute("April");
                textWriter.WriteValue(Statistics.years[year]["4"]);
                // Maj
                textWriter.WriteStartAttribute("May");
                textWriter.WriteValue(Statistics.years[year]["5"]);
                // Juni
                textWriter.WriteStartAttribute("June");
                textWriter.WriteValue(Statistics.years[year]["6"]);
                // Juli
                textWriter.WriteStartAttribute("July");
                textWriter.WriteValue(Statistics.years[year]["7"]);
                // Augusti
                textWriter.WriteStartAttribute("August");
                textWriter.WriteValue(Statistics.years[year]["8"]);
                // September
                textWriter.WriteStartAttribute("September");
                textWriter.WriteValue(Statistics.years[year]["9"]);
                // Oktober
                textWriter.WriteStartAttribute("October");
                textWriter.WriteValue(Statistics.years[year]["10"]);
                // November
                textWriter.WriteStartAttribute("November");
                textWriter.WriteValue(Statistics.years[year]["11"]);
                // December
                textWriter.WriteStartAttribute("December");
                textWriter.WriteValue(Statistics.years[year]["12"]);
                textWriter.WriteEndElement();
            }
            // Ends the document.
            textWriter.WriteEndDocument();
            // close writer
            textWriter.Close();
        }
    }
}